@@ -1,16 +1,16 @@
 import argparse, sys
 
 import scripts
-from xklb.dl_extract import dl_add, dl_block, dl_download, dl_update
-from xklb.fs_actions import filesystem, listen, read, view, watch
+from xklb.consts import SC
+from xklb.dl_extract import dl_block, dl_download
 from xklb.fs_extract import main as fs_add
+from xklb.play_actions import filesystem, listen, read, view, watch
 from xklb.playback import playback_next, playback_now, playback_pause, playback_stop
-from xklb.subtitle import main as subtitle
+from xklb.stats import dlstatus, playlists
 from xklb.tabs_actions import tabs
 from xklb.tabs_extract import tabs_add
-from xklb.tube_actions import tube_list, tube_listen, tube_watch
 from xklb.tube_extract import tube_add, tube_update
-from xklb.utils import SC, log
+from xklb.utils import log
 
 
 def usage() -> str:
@@ -18,9 +18,8 @@ def usage() -> str:
 
     local media subcommands:
       fsadd [extract, xr]          Create a local media database; Add folders
-      subtitle [sub]               Find subtitles for local media
-      listen [lt]                  Listen to local media
-      watch [wt]                   Watch local media
+      listen [lt]                  Listen to local and online media
+      watch [wt]                   Watch local and online media
       read [books, docs]           Read books
       view [see, look]             View images
       filesystem [fs]              Browse files
@@ -31,15 +30,15 @@ def usage() -> str:
     online media subcommands:
       tubeadd [ta, xt]             Create a tube database; Add playlists
       tubeupdate [tu]              Add new videos from saved playlists
-      tubelist [playlists]         List added playlists
-      tubewatch [tw, entries]      Watch the tube
-      tubelisten [tl]              Listen to the tube
 
-    download subcommands (largely untested, under construction):
-      dladd [da]                   Create a download database; Add URLs
-      dlupdate [du]                Add new videos from saved playlists
+    download subcommands:
       download [dl]                Download media
-      block                        Prevent downloading from specific channels
+      block [bl]                   Prevent downloading specific URLs
+      merge-online-local           Merge local and online metadata
+
+    statistics subcommands:
+      playlists [pl, folders]      List added playlists
+      dlstatus [ds]                Show download status
 
     playback subcommands:
       now                          Print what is currently playing
@@ -71,40 +70,47 @@ def lb(args=None) -> None:
     subparsers = parser.add_subparsers()
     subp_extract = subparsers.add_parser("fsadd", aliases=["xr", "extract"], add_help=False)
     subp_extract.set_defaults(func=fs_add)
-    subp_subtitle = subparsers.add_parser("subtitle", aliases=["sub"], add_help=False)
-    subp_subtitle.set_defaults(func=subtitle)
 
-    subp_listen = subparsers.add_parser(SC.listen, aliases=["lt"], add_help=False)
+    subp_listen = subparsers.add_parser(SC.listen, aliases=["lt", "tubelisten", "tl"], add_help=False)
     subp_listen.set_defaults(func=listen)
-    subp_watch = subparsers.add_parser(SC.watch, aliases=["wt"], add_help=False)
+    subp_watch = subparsers.add_parser(SC.watch, aliases=["wt", "tubewatch", "tw", "entries"], add_help=False)
     subp_watch.set_defaults(func=watch)
-    subp_filesystem = subparsers.add_parser(SC.filesystem, aliases=["fs"], add_help=False)
-    subp_filesystem.set_defaults(func=filesystem)
+
     subp_read = subparsers.add_parser(SC.read, aliases=["text", "books", "docs"], add_help=False)
     subp_read.set_defaults(func=read)
     subp_view = subparsers.add_parser(SC.view, aliases=["image", "see", "look"], add_help=False)
     subp_view.set_defaults(func=view)
 
+    subp_filesystem = subparsers.add_parser(SC.filesystem, aliases=["fs"], add_help=False)
+    subp_filesystem.set_defaults(func=filesystem)
+
     subp_bigdirs = subparsers.add_parser("bigdirs", aliases=["largefolders", "large_folders"], add_help=False)
     subp_bigdirs.set_defaults(func=scripts.large_folders)
-
     subp_dedupe = subparsers.add_parser("dedupe", add_help=False)
     subp_dedupe.set_defaults(func=scripts.deduplicate_music)
-
     subp_christen = subparsers.add_parser("christen", add_help=False)
     subp_christen.set_defaults(func=scripts.rename_invalid_files)
+    subp_dedupe_local = subparsers.add_parser("merge-online-local", add_help=False)
+    subp_dedupe_local.set_defaults(func=scripts.merge_online_local)
+    subp_optimize = subparsers.add_parser("optimize", add_help=False)
+    subp_optimize.set_defaults(func=scripts.optimize_db)
 
-    subp_tubelist = subparsers.add_parser("tubelist", aliases=["playlist", "playlists"], add_help=False)
-    subp_tubelist.set_defaults(func=tube_list)
-    subp_tubeadd = subparsers.add_parser("tubeadd", aliases=["ta", "xt"], add_help=False)
+    subp_tubeadd = subparsers.add_parser("tubeadd", aliases=["dladd", "ta", "da", "xt"], add_help=False)
     subp_tubeadd.set_defaults(func=tube_add)
-    subp_tubeupdate = subparsers.add_parser("tubeupdate", aliases=["tu"], add_help=False)
+    subp_tubeupdate = subparsers.add_parser("tubeupdate", aliases=["dlupdate", "tu", "du"], add_help=False)
     subp_tubeupdate.set_defaults(func=tube_update)
 
-    subp_tubewatch = subparsers.add_parser(SC.tubewatch, aliases=["tw", "tube", "entries"], add_help=False)
-    subp_tubewatch.set_defaults(func=tube_watch)
-    subp_tubelisten = subparsers.add_parser(SC.tubelisten, aliases=["tl"], add_help=False)
-    subp_tubelisten.set_defaults(func=tube_listen)
+    subp_download = subparsers.add_parser("download", aliases=["dl"], add_help=False)
+    subp_download.set_defaults(func=dl_download)
+    subp_block = subparsers.add_parser("block", aliases=["bl"], add_help=False)
+    subp_block.set_defaults(func=dl_block)
+
+    subp_playlist = subparsers.add_parser(
+        "playlists", aliases=["playlist", "tubelist", "pl", "folders"], add_help=False
+    )
+    subp_playlist.set_defaults(func=playlists)
+    subp_dlstatus = subparsers.add_parser("dlstatus", aliases=["ds"], add_help=False)
+    subp_dlstatus.set_defaults(func=dlstatus)
 
     subp_playback_now = subparsers.add_parser("now", add_help=False)
     subp_playback_now.set_defaults(func=playback_now)
@@ -120,15 +126,6 @@ def lb(args=None) -> None:
     subp_tabs = subparsers.add_parser("tabs", aliases=["tabswatch", "tb"], add_help=False)
     subp_tabs.set_defaults(func=tabs)
 
-    subp_dladd = subparsers.add_parser("dladd", aliases=["da"], add_help=False)
-    subp_dladd.set_defaults(func=dl_add)
-    subp_dlupdate = subparsers.add_parser("dlupdate", aliases=["du"], add_help=False)
-    subp_dlupdate.set_defaults(func=dl_update)
-    subp_download = subparsers.add_parser("download", aliases=["dl"], add_help=False)
-    subp_download.set_defaults(func=dl_download)
-    subp_block = subparsers.add_parser("block", add_help=False)
-    subp_block.set_defaults(func=dl_block)
-
     parser.add_argument("--version", "-V", action="store_true")
     args, _unk = parser.parse_known_args(args)
     if args.version:
