@@ -7,29 +7,21 @@ from random import randrange
 from shlex import join, quote
 from shutil import which
 from time import sleep
-from typing import List, Tuple, Union
+from typing import List, Optional, Tuple, Union
 
 import screeninfo
 from rich.prompt import Confirm
 from tabulate import tabulate
 
-from xklb import gui, paths, utils
-from xklb.utils import (
-    DEFAULT_MULTIPLE_PLAYBACK,
-    SC,
-    SQLITE_PARAM_LIMIT,
-    cmd,
-    cmd_interactive,
-    human_time,
-    log,
-    os_bg_kwargs,
-)
+from xklb import consts, gui, utils
+from xklb.consts import SC
+from xklb.utils import cmd, cmd_interactive, human_time, log, os_bg_kwargs
 
 
 def generic_player(args) -> List[str]:
     if platform.system() == "Linux":
         player = ["xdg-open"]
-    elif any([p in platform.system() for p in ["Windows", "_NT-", "MSYS"]]):
+    elif any([p in platform.system() for p in ("Windows", "_NT-", "MSYS")]):
         if shutil.which("cygstart"):
             player = ["cygstart"]
         else:
@@ -60,7 +52,7 @@ def calculate_duration(args, m) -> Tuple[int, int]:
     return start, end
 
 
-def find_xdg_application(media_file) -> Union[str, None]:
+def find_xdg_application(media_file) -> Optional[str]:
     mimetype = cmd("xdg-mime", "query", "filetype", media_file).stdout
     default_application = cmd("xdg-mime", "query", "default", mimetype).stdout
     player_path = which(default_application.replace(".desktop", ""))
@@ -75,19 +67,19 @@ def parse(args, m=None, media_file=None) -> List[str]:
         player = args.player
         args.player_need_sleep = False
 
-    elif args.action in [SC.read] and media_file:
+    elif args.action in (SC.read) and media_file:
         player_path = find_xdg_application(media_file)
         log.info(player_path)
 
     elif mpv:
         args.player_need_sleep = False
         player = [mpv]
-        if args.action in [SC.listen, SC.tubelisten]:
+        if args.action in (SC.listen):
             player.extend([f"--input-ipc-server={args.mpv_socket}", "--no-video", "--keep-open=no", "--really-quiet"])
-        elif args.action in [SC.watch, SC.tubewatch]:
+        elif args.action in (SC.watch):
             player.extend(["--force-window=yes", "--really-quiet"])
 
-        if args.action in [SC.tubelisten, SC.tubewatch]:
+        if media_file and media_file.startswith("http"):
             player.extend(["--script-opts=ytdl_hook-try_ytdl_first=yes"])
 
         if not args.multiple_playback:
@@ -99,13 +91,13 @@ def parse(args, m=None, media_file=None) -> List[str]:
         if args.crop:
             player.extend(["--panscan=1.0"])
 
-        if m and args.action in [SC.watch, SC.listen]:
+        if m and args.action in (SC.watch, SC.listen):
             start, end = calculate_duration(args, m)
             if end != 0:
                 if start != 0:
-                    player.extend([f"--start={int(start)}", "--no-save-position-on-quit"])
+                    player.extend([f"--start={start}", "--no-save-position-on-quit"])
                 if end != m["duration"]:
-                    player.extend([f"--end={int(end)}"])
+                    player.extend([f"--end={end}"])
 
         if m and args.action == SC.watch:
             if m["subtitle_count"] > 0:
@@ -144,8 +136,8 @@ def mv_to_keep_folder(args, media_file: str) -> None:
 def moved_media(args, moved_files: Union[str, list], base_from, base_to) -> int:
     moved_files = utils.conform(moved_files)
     modified_row_count = 0
-    if len(moved_files) > 0:
-        df_chunked = utils.chunks(moved_files, SQLITE_PARAM_LIMIT)
+    if moved_files:
+        df_chunked = utils.chunks(moved_files, consts.SQLITE_PARAM_LIMIT)
         for l in df_chunked:
             with args.db.conn:
                 cursor = args.db.conn.execute(
@@ -164,8 +156,8 @@ def moved_media(args, moved_files: Union[str, list], base_from, base_to) -> int:
 def mark_media_watched(args, files) -> int:
     files = utils.conform(files)
     modified_row_count = 0
-    if len(files) > 0:
-        df_chunked = utils.chunks(files, SQLITE_PARAM_LIMIT)
+    if files:
+        df_chunked = utils.chunks(files, consts.SQLITE_PARAM_LIMIT)
         for l in df_chunked:
             with args.db.conn:
                 cursor = args.db.conn.execute(
@@ -186,13 +178,13 @@ def mark_media_deleted(args, paths) -> int:
     paths = utils.conform(paths)
 
     modified_row_count = 0
-    if len(paths) > 0:
-        df_chunked = utils.chunks(paths, SQLITE_PARAM_LIMIT)
+    if paths:
+        df_chunked = utils.chunks(paths, consts.SQLITE_PARAM_LIMIT)
         for l in df_chunked:
             with args.db.conn:
                 cursor = args.db.conn.execute(
-                    """update media
-                    set is_deleted=1
+                    f"""update media
+                    set time_deleted={consts.NOW}
                     where path in ("""
                     + ",".join(["?"] * len(l))
                     + ")",
@@ -206,7 +198,7 @@ def mark_media_deleted(args, paths) -> int:
 def delete_media(args, paths) -> int:
     paths = utils.conform(paths)
     for p in paths:
-        if len(args.prefix) > 0:
+        if args.prefix:
             Path(p).unlink(missing_ok=True)
         else:
             utils.trash(p)
@@ -231,7 +223,7 @@ def post_act(args, media_file: str, action=None) -> None:
     if action == "keep":
         return
 
-    if args.action in [SC.tubelisten, SC.tubewatch]:
+    if media_file.startswith("http"):
         if action == "remove":
             mark_media_deleted(args, media_file)
         elif action == "ask":
@@ -239,8 +231,7 @@ def post_act(args, media_file: str, action=None) -> None:
                 mark_media_deleted(args, media_file)
         else:
             raise Exception("Unrecognized action", action)
-
-    if args.action in [SC.listen, SC.watch]:
+    else:
         if action == "softdelete":
             mark_media_deleted(args, media_file)
 
@@ -279,10 +270,11 @@ def override_sort(sort_expression: str) -> str:
 def get_ordinal_media(args, path: str) -> str:
     # TODO: maybe try https://dba.stackexchange.com/questions/43415/algorithm-for-finding-the-longest-prefix
 
-    similar_videos = []
-    candidate = deepcopy(path)
+    columns = [c.name for c in args.db["media"].columns]
 
     total_media = args.db.execute("select count(*) val from media").fetchone()[0]
+    candidate = deepcopy(path)
+    similar_videos = []
     while len(similar_videos) < 2:
         remove_groups = re.split(r"([\W]+|\s+|Ep\d+|x\d+|\.\d+)", candidate)
         log.debug(remove_groups)
@@ -295,14 +287,14 @@ def get_ordinal_media(args, path: str) -> str:
         new_candidate = candidate[: -len(remove_chars)]
         log.debug(f"Matches for '{new_candidate}':")
 
-        if candidate in ["" or new_candidate]:
+        if candidate in ("" or new_candidate):
             return path
 
         candidate = new_candidate
         query = f"""SELECT path FROM {args.table}
             WHERE 1=1
                 and path like :candidate
-                {'and is_deleted=0' if args.action in [SC.listen, SC.watch] else ''}
+                {'and time_deleted=0' if 'time_deleted' in columns else ''}
                 {'' if (args.play_in_order >= 3) else (args.sql_filter or '')}
             ORDER BY path
             LIMIT 1000
@@ -329,7 +321,7 @@ def get_ordinal_media(args, path: str) -> str:
     return similar_videos[0]
 
 
-def watch_chromecast(args, m: dict, subtitles_file=None) -> Union[subprocess.CompletedProcess, None]:
+def watch_chromecast(args, m: dict, subtitles_file=None) -> Optional[subprocess.CompletedProcess]:
     if "vlc" in args.player:
         catt_log = cmd(
             "vlc",
@@ -342,14 +334,14 @@ def watch_chromecast(args, m: dict, subtitles_file=None) -> Union[subprocess.Com
             m["path"],
         )
     else:
-        if args.action in [SC.watch, SC.listen]:
+        if args.action in (SC.watch, SC.listen):
             catt_log = cmd(
                 "catt",
                 "-d",
                 args.chromecast_device,
                 "cast",
                 "-s",
-                subtitles_file if subtitles_file else paths.FAKE_SUBTITLE,
+                subtitles_file if subtitles_file else consts.FAKE_SUBTITLE,
                 m["path"],
             )
         else:
@@ -360,12 +352,12 @@ def watch_chromecast(args, m: dict, subtitles_file=None) -> Union[subprocess.Com
     return catt_log
 
 
-def listen_chromecast(args, m: dict) -> Union[subprocess.CompletedProcess, None]:
-    Path(paths.CAST_NOW_PLAYING).write_text(m["path"])
-    Path(paths.FAKE_SUBTITLE).touch()
+def listen_chromecast(args, m: dict) -> Optional[subprocess.CompletedProcess]:
+    Path(consts.CAST_NOW_PLAYING).write_text(m["path"])
+    Path(consts.FAKE_SUBTITLE).touch()
     if args.with_local:
         cast_process = subprocess.Popen(
-            ["catt", "-d", args.chromecast_device, "cast", "-s", paths.FAKE_SUBTITLE, m["path"]], **os_bg_kwargs()
+            ["catt", "-d", args.chromecast_device, "cast", "-s", consts.FAKE_SUBTITLE, m["path"]], **os_bg_kwargs()
         )
         sleep(0.974)  # imperfect lazy sync; I use keyboard shortcuts to send `set speed` commands to mpv for resync
         # if pyChromecast provides a way to sync accurately that would be very interesting to know; I have not researched it
@@ -373,10 +365,10 @@ def listen_chromecast(args, m: dict) -> Union[subprocess.CompletedProcess, None]
         catt_log = utils.Pclose(cast_process)  # wait for chromecast to stop (you can tell any chromecast to pause)
         sleep(3.0)  # give chromecast some time to breathe
     else:
-        if args.action in [SC.watch, SC.listen]:
-            catt_log = cmd("catt", "-d", args.chromecast_device, "cast", "-s", paths.FAKE_SUBTITLE, m["path"])
-        else:  # args.action in [SC.tubewatch, SC.tubelisten]:
+        if m["path"].startswith("http"):
             catt_log = args.cc.play_url(m["path"], resolve=True, block=True)
+        else:  #  local file
+            catt_log = cmd("catt", "-d", args.chromecast_device, "cast", "-s", consts.FAKE_SUBTITLE, m["path"])
 
     return catt_log
 
@@ -400,12 +392,12 @@ def socket_play(args, m: dict) -> None:
         return
 
     play_opts = f"start={start},save-position-on-quit=no"
-    if args.action in [SC.listen, SC.tubelisten]:
+    if args.action in (SC.listen):
         play_opts += ",video=no,really-quiet=yes"
-    elif args.action in [SC.watch, SC.tubewatch]:
+    elif args.action in (SC.watch):
         play_opts += ",fullscreen=yes,force-window=yes,really-quiet=yes"
 
-    if args.action in [SC.tubelisten, SC.tubewatch]:
+    if m["path"].startswith("http"):
         play_opts += ",script-opts=ytdl_hook-try_ytdl_first=yes"
 
     f = m["path"].replace("\\", "\\\\").replace('"', '\\"').replace("\n", "\\n")
@@ -475,9 +467,9 @@ def get_multiple_player_template(args) -> List[str]:
     if args.screen_name:
         displays = get_display_by_name(displays, args.screen_name)
 
-    if args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) == 1:
+    if args.multiple_playback == consts.DEFAULT_MULTIPLE_PLAYBACK and len(displays) == 1:
         args.multiple_playback = 2
-    elif args.multiple_playback == DEFAULT_MULTIPLE_PLAYBACK and len(displays) >= 2:
+    elif args.multiple_playback == consts.DEFAULT_MULTIPLE_PLAYBACK and len(displays) >= 2:
         args.multiple_playback = len(displays)
     elif args.multiple_playback < len(displays):
         # play videos on supporting screens but not active one
@@ -500,26 +492,45 @@ def get_multiple_player_template(args) -> List[str]:
     return players
 
 
-def multiple_player(args, media) -> None:
-    def open_player(template_args, m):
-        print(m["path"])
-        mp_args = ["--window-scale=1", "--no-border", "--no-keepaspect-window"]
-        return subprocess.Popen([*args.player, *mp_args, *template_args, "--", m["path"]], **os_bg_kwargs())
+def _create_player(args, window_geometry, media):
+    m = media.pop()
+    print(m["path"])
+    mp_args = ["--window-scale=1", "--no-border", "--no-keepaspect-window"]
+    return {
+        **m,
+        "process": subprocess.Popen([*args.player, *mp_args, *window_geometry, "--", m["path"]], **os_bg_kwargs()),
+    }
 
+
+def gui_post_act(args, media, m):
+    if args.post_action in ("ask", "askkeep"):
+        r = gui.askkeep(m["path"], len(media))
+        if r == "DELETE":
+            post_act(args, m["path"], action="delete")
+        elif r == "KEEP" and args.post_action == "ask":
+            post_act(args, m["path"], action="keep")
+        elif r == "KEEP" and args.post_action == "askkeep":
+            mv_to_keep_folder(args, m["path"])
+        else:
+            raise Exception("I did not quite catch that... what did you say?")
+    else:
+        post_act(args, m["path"])
+
+
+def multiple_player(args, media) -> None:
     args.player = parse(args)
 
     template = get_multiple_player_template(args)
     players = []
 
-    try:
+    try:  # pylint: disable=too-many-nested-blocks
         while media:
-            for t_idx, t in enumerate(template):
+            for t_idx, player_hole in enumerate(template):
                 try:
                     m = players[t_idx]
                 except IndexError:
                     log.debug("%s IndexError", t_idx)
-                    m = media.pop()
-                    players.append({**m, "process": open_player(t, m)})
+                    players.append(_create_player(args, player_hole, media))
                 else:
                     log.debug("%s Check if still running", t_idx)
                     if m["process"].poll() is not None:
@@ -530,35 +541,23 @@ def multiple_player(args, media) -> None:
                             if not args.ignore_errors:
                                 exit(r.returncode)
 
-                        if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
-                            if args.post_action in ["ask", "askkeep"]:
-                                r = gui.askkeep(m["path"], len(media))
-                                if r == "DELETE":
-                                    post_act(args, m["path"], action="delete")
-                                elif r == "KEEP" and args.post_action == "ask":
-                                    post_act(args, m["path"], action="keep")
-                                elif r == "KEEP" and args.post_action == "askkeep":
-                                    mv_to_keep_folder(args, m["path"])
-                                else:
-                                    raise Exception("I did not quite catch that... what did you say?")
-                            else:
-                                post_act(args, m["path"])
+                        if args.action in (SC.listen, SC.watch):
+                            gui_post_act(args, media, m)
 
-                        m = media.pop()
-                        players[t_idx] = {**m, "process": open_player(t, m)}
+                        players[t_idx] = _create_player(args, player_hole, media)
 
             log.debug("-- A dragon slumbers over its hoard of %s media --", len(media))
-            sleep(0.2)
+            sleep(0.2)  # I don't know if this is necessary but may as well~~
     finally:
         for m in players:
             m["process"].kill()
 
 
 def local_player(args, m, media_file) -> subprocess.CompletedProcess:
-    if system() == "Windows" or args.action in [SC.watch, SC.tubewatch]:
+    if system() == "Windows" or args.action in (SC.watch):
         r = cmd(*args.player, media_file, strict=False)
-    else:  # args.action in [SC.listen, SC.tubelisten]
-        r = cmd_interactive(*args.player, media_file, strict=False)
+    else:  # args.action in (SC.listen)
+        r = cmd_interactive(*args.player, media_file)
 
     if args.player_need_sleep:
         if hasattr(m, "duration"):
@@ -571,43 +570,43 @@ def local_player(args, m, media_file) -> subprocess.CompletedProcess:
 
 
 def printer(args, query, bindings) -> None:
+    columns = [c.name for c in args.db["media"].columns]
     if "a" in args.print:
         query = f"""select
             "Aggregate" as path
-            {', sum(duration) duration' if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch, SC.read] else ''}
-            {', avg(duration) avg_duration' if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch, SC.read] else ''}
-            {', sparseness' if args.action == SC.filesystem else ''}
-            {', sum(size) size' if args.action != SC.tabs else ''}
+            {', sum(duration) duration' if 'duration' in columns else ''}
+            {', avg(duration) avg_duration' if 'duration' in columns else ''}
+            {', sparseness' if 'sparseness' in columns else ''}
+            {', sum(size) size' if 'size' in columns else ''}
             , count(*) count
             {', ' + ', '.join([f'sum({c}) sum_{c}' for c in args.cols]) if args.cols else ''}
             {', ' + ', '.join([f'avg({c}) avg_{c}' for c in args.cols]) if args.cols else ''}
         from ({query}) """
 
-    db_resp = list(args.db.query(query, bindings))
+    media = list(args.db.query(query, bindings))
 
     if "v" in args.print and Path(args.watch_later_directory).exists():
-        db_resp = utils.mpv_enrich2(args, db_resp)
+        media = utils.mpv_enrich2(args, media)
 
     if args.verbose >= 2 and args.cols and "*" in args.cols:
         breakpoint()
 
-    if len(db_resp) == 0:
-        print("No media found")
-        exit(2)
+    if not media:
+        utils.no_media_found()
 
     if "d" in args.print:
-        mark_media_deleted(args, list(map(operator.itemgetter("path"), db_resp)))
+        mark_media_deleted(args, list(map(operator.itemgetter("path"), media)))
         if not "f" in args.print:
-            return print(f"Removed {len(db_resp)} metadata records")
+            return print(f"Removed {len(media)} metadata records")
 
     if "w" in args.print:
-        marked = mark_media_watched(args, list(map(operator.itemgetter("path"), db_resp)))
+        marked = mark_media_watched(args, list(map(operator.itemgetter("path"), media)))
         if not "f" in args.print:
             return print(f"Marked {marked} metadata records as watched")
 
     if "f" in args.print:
         if args.limit == 1:
-            f = db_resp[0]["path"]
+            f = media[0]["path"]
             if not Path(f).exists():
                 mark_media_deleted(args, f)
                 return printer(args, query, bindings)
@@ -616,7 +615,7 @@ def printer(args, query, bindings) -> None:
             if not args.cols:
                 args.cols = ["path"]
 
-            selected_cols = [{k: d.get(k, None) for k in args.cols} for d in db_resp]
+            selected_cols = [{k: d.get(k, None) for k in args.cols} for d in media]
             virtual_csv = StringIO()
             wr = csv.writer(virtual_csv, quoting=csv.QUOTE_NONE)
             wr = csv.DictWriter(virtual_csv, fieldnames=args.cols)
@@ -629,9 +628,9 @@ def printer(args, query, bindings) -> None:
                 else:
                     print(line.strip())
             if args.moved:
-                moved_media(args, list(map(operator.itemgetter("path"), db_resp)), *args.moved)
+                moved_media(args, list(map(operator.itemgetter("path"), media)), *args.moved)
     else:
-        tbl = deepcopy(db_resp)
+        tbl = deepcopy(media)
         utils.col_resize(tbl, "path", 22)
         utils.col_resize(tbl, "title", 18)
 
@@ -639,23 +638,16 @@ def printer(args, query, bindings) -> None:
         utils.col_duration(tbl, "duration")
         utils.col_duration(tbl, "avg_duration")
 
-        for t in [
-            "time_modified",
-            "time_created",
-            "time_played",
-            "time_valid",
-            "time_partial_first",
-            "time_partial_last",
-        ]:
+        for t in consts.TIME_COLUMNS:
             utils.col_naturaldate(tbl, t)
 
-        print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))  # type: ignore
+        print(tabulate(tbl, tablefmt="fancy_grid", headers="keys", showindex=False))
 
-        if args.action in [SC.listen, SC.watch, SC.tubelisten, SC.tubewatch]:
-            if len(db_resp) >= 2:
-                print(f"{len(db_resp)} media" + (f" (limited to {args.limit})" if args.limit else ""))
+        if "duration" in columns:
+            if len(media) >= 2:
+                print(f"{len(media)} media" + (f" (limited to {args.limit})" if args.limit else ""))
 
-            duration = sum(map(lambda m: m.get("duration") or 0, db_resp))
+            duration = sum(map(lambda m: m.get("duration") or 0, media))
             duration = human_time(duration)
             if not "a" in args.print:
                 print("Total duration:", duration)
