@@ -5,7 +5,7 @@ from typing import List
 from urllib.parse import urlparse
 
 from xklb import db, player, utils
-from xklb.paths import Frequency, sanitize_url
+from xklb.consts import Frequency, sanitize_url
 from xklb.utils import argparse_enum, log
 
 
@@ -32,9 +32,6 @@ def parse_args() -> argparse.Namespace:
 
 """,
     )
-    parser.add_argument("database", nargs="?", default="tabs.db")
-    parser.add_argument("paths", nargs="+")
-    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
     parser.add_argument(
         "--frequency",
         "--freqency",
@@ -46,8 +43,11 @@ def parse_args() -> argparse.Namespace:
     )
     parser.add_argument("--category", "-c", help=argparse.SUPPRESS)
     parser.add_argument("--no-sanitize", "-s", action="store_true", help="Don't sanitize some common URL parameters")
-
     parser.add_argument("-v", "--verbose", action="count", default=0)
+    parser.add_argument("--db", "-db", help=argparse.SUPPRESS)
+
+    parser.add_argument("database", nargs="?", default="tabs.db")
+    parser.add_argument("paths", nargs="+")
     args = parser.parse_args()
 
     if args.db:
@@ -81,7 +81,7 @@ def get_new_paths(args) -> List[str]:
     except Exception:
         pass
     else:
-        if len(existing) > 0:
+        if existing:
             print(f"Updating frequency for {len(existing)} existing paths")
             player.mark_media_deleted(args, list(existing))
 
@@ -97,10 +97,10 @@ def extract_url_metadata(args, path: str) -> dict:
         hostname=hostname,
         frequency=args.frequency.value,
         category=args.category,
-        time_created=int(datetime.utcnow().timestamp()),
+        time_created=int(datetime.now().timestamp()),
         time_played=0,
         play_count=0,
-        is_deleted=0,
+        time_deleted=0,
     )
 
 
@@ -110,4 +110,4 @@ def tabs_add(args=None) -> None:
     args = parse_args()
 
     tabs = [extract_url_metadata(args, path) for path in get_new_paths(args)]
-    args.db["media"].insert_all(tabs, pk="path", alter=True, replace=True)  # type: ignore
+    args.db["media"].insert_all(tabs, pk="path", alter=True, replace=True)
