@@ -1,11 +1,11 @@
 import argparse, enum, functools, hashlib, logging, math, multiprocessing, os, platform, re, signal, subprocess, sys, textwrap
+from ast import literal_eval
 from collections.abc import Iterable
-from datetime import datetime, timedelta
+from datetime import datetime, timedelta, timezone
 from functools import wraps
 from pathlib import Path
 from shutil import which
-from types import SimpleNamespace
-from typing import Any, Dict, Generator, List, Union
+from typing import Any, Dict, Generator, List, Optional, Union
 
 import humanize
 from IPython.core import ultratb
@@ -13,6 +13,8 @@ from IPython.terminal.debugger import TerminalPdb
 from pychromecast import discovery
 from rich.logging import RichHandler
 
+from xklb import consts
+
 try:
     import ipdb
 except ModuleNotFoundError:
@@ -20,12 +22,6 @@ except ModuleNotFoundError:
 else:
     sys.breakpointhook = ipdb.set_trace
 
-SQLITE_PARAM_LIMIT = 32765
-DEFAULT_PLAY_QUEUE = 120
-DEFAULT_MULTIPLE_PLAYBACK = -1
-CPU_COUNT = int(os.cpu_count() or 4)
-PYTEST_RUNNING = "pytest" in sys.modules
-
 
 def exit_nicely(_signal, _frame):
     print("\nExiting... (Ctrl+C)\n")
@@ -35,15 +31,9 @@ def exit_nicely(_signal, _frame):
 signal.signal(signal.SIGINT, exit_nicely)
 
 
-class SC:
-    watch = "watch"
-    listen = "listen"
-    filesystem = "filesystem"
-    tubewatch = "tubewatch"
-    tubelisten = "tubelisten"
-    tabs = "tabs"
-    read = "read"
-    view = "view"
+def no_media_found():
+    print("No media found")
+    exit(2)
 
 
 def with_timeout(timeout):
@@ -157,7 +147,7 @@ def cmd(*command, strict=True, cwd=None, quiet=True, interactive=False, **kwargs
 
     def print_std(r_std):
         s = filter_output(r_std)
-        if not quiet and len(s) > 0:
+        if not quiet and s:
             print(s)
         return s
 
@@ -211,7 +201,7 @@ def remove_text_inside_brackets(text: str, brackets="()[]") -> str:  # thanks @j
 def get_ip_of_chromecast(device_name):
     cast_infos, browser = discovery.discover_listed_chromecasts(friendly_names=[device_name])
     browser.stop_discovery()
-    if len(cast_infos) == 0:
+    if not cast_infos:
         print("Target chromecast device not found")
         exit(53)
 
@@ -236,7 +226,7 @@ def mpv_enrich2(args, media) -> List[dict]:
     paths = list(Path(args.watch_later_directory).glob("*"))
     filtered_list = [
         {
-            **md5s.get(p.stem),  # type: ignore
+            **(md5s.get(p.stem) or {}),
             "time_partial_first": int(p.stat().st_ctime),
             "time_partial_last": int(p.stat().st_mtime),
         }
@@ -247,13 +237,24 @@ def mpv_enrich2(args, media) -> List[dict]:
     return sorted(filtered_list, key=lambda m: m.get("time_partial_first") or 0, reverse=False)
 
 
-def dict_filter_bool(kwargs) -> dict:
-    return {k: v for k, v in kwargs.items() if v}
+def dict_filter_bool(kwargs, keep_0=True) -> Optional[dict]:
+    if keep_0:
+        filtered_dict = {k: v for k, v in kwargs.items() if v is not None and v != "" and v is not False}
+    else:
+        filtered_dict = {k: v for k, v in kwargs.items() if v}
+
+    if len(filtered_dict) == 0:
+        return None
+    return filtered_dict
 
 
-def cmd_interactive(*cmd, **kwargs) -> subprocess.CompletedProcess:
-    return_code = os.spawnvpe(os.P_WAIT, cmd[0], cmd, os.environ)
-    return subprocess.CompletedProcess(cmd, return_code)
+def list_dict_filter_bool(media: List[dict]) -> List[dict]:
+    return [d for d in [dict_filter_bool(d) for d in media] if d]
+
+
+def cmd_interactive(*command) -> subprocess.CompletedProcess:
+    return_code = os.spawnvpe(os.P_WAIT, command[0], command, os.environ)
+    return subprocess.CompletedProcess(command, return_code)
 
 
 def Pclose(process) -> subprocess.CompletedProcess:
@@ -292,23 +293,23 @@ def divisor_gen(n: float) -> Generator:
 _RE_COMBINE_WHITESPACE = re.compile(r"\s+")
 
 
-def combine(*list_) -> Union[str, None]:
+def combine(*list_) -> Optional[str]:
     list_ = conform(list_)
-    if len(list_) == 0:
+    if not list_:
         return None
 
     no_comma = sum([str(s).split(",") for s in list_], [])
     no_semicolon = sum([s.split(";") for s in no_comma], [])
     no_double_space = [_RE_COMBINE_WHITESPACE.sub(" ", s).strip() for s in no_semicolon]
-    no_unknown = [x for x in no_double_space if x.lower() not in ["unknown", "none", "und", ""]]
+    no_unknown = [x for x in no_double_space if x.lower() not in ("unknown", "none", "und", "")]
 
     no_duplicates = list(dict.fromkeys(no_unknown))
     return ";".join(no_duplicates)
 
 
-def safe_unpack(*list_, idx=0) -> Union[Any, None]:
+def safe_unpack(*list_, idx=0) -> Optional[Any]:
     list_ = conform(list_)
-    if len(list_) == 0:
+    if not list_:
         return None
 
     try:
@@ -317,16 +318,10 @@ def safe_unpack(*list_, idx=0) -> Union[Any, None]:
         return None
 
 
-try:
-    TERMINAL_SIZE = os.get_terminal_size()
-except Exception:
-    TERMINAL_SIZE = SimpleNamespace(columns=80, lines=60)
-
-
 def col_resize(tbl: List[Dict], col: str, size=10) -> List[Dict]:
     for idx, _d in enumerate(tbl):
         if tbl[idx].get(col) is not None:
-            tbl[idx][col] = textwrap.fill(tbl[idx][col], max(10, int(size * (TERMINAL_SIZE.columns / 80))))
+            tbl[idx][col] = textwrap.fill(tbl[idx][col], max(10, int(size * (consts.TERMINAL_SIZE.columns / 80))))
 
     return tbl
 
@@ -334,7 +329,7 @@ def col_resize(tbl: List[Dict], col: str, size=10) -> List[Dict]:
 def col_naturaldate(tbl: List[Dict], col: str) -> List[Dict]:
     for idx, _d in enumerate(tbl):
         if tbl[idx].get(col) is not None:
-            tbl[idx][col] = humanize.naturaldate(datetime.fromtimestamp(tbl[idx][col]))
+            tbl[idx][col] = humanize.naturaldate(datetime.fromtimestamp(int(tbl[idx][col]), tz=timezone.utc))
 
     return tbl
 
@@ -342,13 +337,16 @@ def col_naturaldate(tbl: List[Dict], col: str) -> List[Dict]:
 def col_naturalsize(tbl: List[Dict], col: str) -> List[Dict]:
     for idx, _d in enumerate(tbl):
         if tbl[idx].get(col) is not None:
-            tbl[idx][col] = humanize.naturalsize(tbl[idx][col])
+            if tbl[idx][col] == 0:
+                tbl[idx][col] = None
+            else:
+                tbl[idx][col] = humanize.naturalsize(tbl[idx][col])
 
     return tbl
 
 
-def human_time(seconds) -> Union[str, None]:
-    if seconds is None or math.isnan(seconds):
+def human_time(seconds) -> Optional[str]:
+    if seconds is None or math.isnan(seconds) or seconds == 0:
         return None
     hours = humanize.precisedelta(timedelta(seconds=int(seconds)), minimum_unit="hours", format="%0.0f")
     if len(hours.split(",")) >= 3:
@@ -356,8 +354,8 @@ def human_time(seconds) -> Union[str, None]:
     minutes = humanize.precisedelta(timedelta(seconds=int(seconds)), minimum_unit="minutes", format="%0.0f")
     if len(minutes.split(",")) >= 2:
         return minutes
-    else:
-        return humanize.precisedelta(timedelta(seconds=int(seconds)), minimum_unit="minutes")
+
+    return humanize.precisedelta(timedelta(seconds=int(seconds)), minimum_unit="minutes")
 
 
 def col_duration(tbl: List[Dict], col: str) -> List[Dict]:
@@ -372,10 +370,17 @@ def col_duration(tbl: List[Dict], col: str) -> List[Dict]:
 class argparse_dict(argparse.Action):
     def __call__(self, parser, args, values, option_string=None):
         try:
-            e = ",".join(list(flatten([v.split(" ") for v in values])))
-            d = eval(f"dict({e})")
+            d = {}
+            k_eq_v = list(flatten([val.split(" ") for val in values]))
+            for s in k_eq_v:
+                k, v = s.split("=")
+                if any([sym in v for sym in ("[", "{")]):
+                    d[k] = literal_eval(v)
+                else:
+                    d[k] = v
+
         except ValueError as ex:
-            raise argparse.ArgumentError(self, f'Could not parse argument "{values}" as k1=1 k2=2 format')
+            raise argparse.ArgumentError(self, f'Could not parse argument "{values}" as k1=1 k2=2 format {ex}')
         setattr(args, self.dest, d)
 
 
